;===========================================================================
; This instrument is an attempt at simulating a modular synthesizer.
;===========================================================================
sr=44100
kr=44100
ksmps=1
nchnls=2
zakinit 30,30
;---------------------------------------------------------------------------
;  1. Keyboard Trigger
;  2. Envelope
;  3. Table LFO
;  4. Trigger
;  7. Sequencer
; 10. Table    Audio Oscillator
; 11. Noise    Audio Oscillator
; 12. Triangle Audio Oscillator
; 20. 4-Pole Low-Pass Resonant Filter
; 25. Controlled Amplifier
; 30. Distortion
; 40. Flanger
; 90. Sub-Mix
;100. Mixer

;---------------------------------------------------------------------------
; Single Trigger
;---------------------------------------------------------------------------
          instr  1

idur      init   p3
idurch    init   p4

          zkw    idur, idurch

          endin

;---------------------------------------------------------------------------
; Keyboard Trigger
;---------------------------------------------------------------------------
          instr  2

idur      init   p3
iamp      init   p4
ifqc      =      cpspch(p5)
idurch    init   p6
iampch    init   p7
ifqcch    init   p8

          zkw    idur, idurch
          zkw    iamp, iampch
          zkw    ifqc, ifqcch

          endin

;---------------------------------------------------------------------------
; Duration Sequencer
;---------------------------------------------------------------------------
          instr  3

idur      init   p3
idurtab   init   p4
idurch    init   p5
kstep     init   0

kdur1     table  kstep, idurtab
kdur      =      kdur1/8               ; Make the step smaller.

reset:    timout 0, i(kdur), next1
kstep     =      frac((kstep + 1)/8)*8
kstep2    =      frac((kstep + 7)/8)*8
          reinit reset
next1:    

kdur2     table  kstep2, idurtab
kdur3     =      kdur2/8

          zkw    kdur3, idurch

          endin

;---------------------------------------------------------------------------
; Analog Sequencer
;---------------------------------------------------------------------------
          instr  4

idur      init   p3
iamptab   init   p4
ipchtab   init   p5
idurch    init   p6
iampch    init   p7
ifqcch    init   p8

kdur      zkr    idurch

kamp      oscil  1, 1/kdur/8, iamptab   ; Read the amplitude
kpch      oscil  1, 1/kdur/8, ipchtab   ; Read the pitch
kfqc      =      cpspch(kpch)         ; Convert to frequency

          zkw    kamp, iampch
          zkw    kfqc, ifqcch

          endin

;---------------------------------------------------------------------------
; Envelope
;---------------------------------------------------------------------------
          instr   5

idurch    init    p4
iamp      init    p5
itab1     init    p6
iout      init    p7

kdur     zkr     idurch
kenv     oscil   iamp, 1/kdur, itab1  ;Envelope

          zkw    kenv, iout         ;Write to output

endin

;---------------------------------------------------------------------------
; Table LFO
;---------------------------------------------------------------------------
          instr   6

iamp      init    p4
ifqc      init    p5
itab1     init    p6
ioffset   init    p7
iout      init    p8

koscil    oscil   iamp, ifqc, itab1  ;Oscillator
kout      =       koscil+ioffset

          zkw     kout, iout           ;Mix to output channel

endin

;---------------------------------------------------------------------------
; Table Audio Oscillator
;---------------------------------------------------------------------------
          instr   10

itab1     init    p4
ifqch     init    p5
iout      init    p6

kfqc      zkr     ifqch
asin1     oscil   1, kfqc, itab1  ;Oscillator

          zawm    asin1, iout           ;Mix to output channel

endin

;---------------------------------------------------------------------------
; Noise Audio Oscillator
;---------------------------------------------------------------------------
          instr   11

iout      init    p4

anoyz     rand    1              ;Noise

          zawm    anoyz, iout    ;Mix to output channel

endin

;---------------------------------------------------------------------------
; Triangle Audio Oscillator
;---------------------------------------------------------------------------
         instr   12   

idur     init    p3
iamp     init    p4
ifqch    init    p5
kxn      init    -p4
kmax     init    p4
ksyncm   init    -1
ksyncs   init    -1
ioutch   init    p7

kfqc     zkr     ifqch
kfqcs    =       kfqc*p6
kdx      =       p4*8*kfqc/sr

kxn      =       kxn+kdx                     ;Find next position 
ksmprev  =       ksyncm                      ;Watch master for transition
ksyncm   oscil   1,kfqc,2                    ;from -1 to 1 then reset axn.
kxn      =       ((ksmprev=-1)&&(ksyncm=1) ? -kmax : kxn)

kssprev  =       ksyncs                      ;Watch slave for transition
ksyncs   oscil   1,kfqcs,2                   ;from -1 to 1 then reset axn.
kxn      =       ((kssprev=-1)&&(ksyncs=1) ? -kmax : kxn)

axn      =       kxn

         zaw     axn, ioutch

         endin

;---------------------------------------------------------------------------
; Feather Fractal Audio Oscillator
;---------------------------------------------------------------------------
instr 13

  kampenv linseg 0, .01, p4, p3-.02, p4, .01, 0

  ih init p5
  ia init -.48
  ic init 2-2*ia
  ib init .93
  ax init 3
  ay init 0
  aw init ia*3+ic*9/10
  ifqc = cpspch(p6)
  idur init p3

  kclknew init -1

; Amplitude envelope
;  kampenv linseg 0, .01, p4, p3-.02, p4, .01, 0
;  kampenv  oscil p4, 1/idur, 5
  kclkold =     kclknew
  kclknew oscil 1, ifqc/2, 2

  if (kclkold == kclknew) goto next

    axold = ax
    ayold = ay

    az = ax
    ax = ib*ay+aw
    au = ax*ax
    aw = ia*ax+ic*au/(1+au)
    ay = aw-az

    axsig = axold
    aysig = ayold
    adx   = (ax-axold)*ifqc/sr
    ady   = (ay-ayold)*ifqc/sr
  next:

  axsig = axsig+adx
  aysig = aysig+ady

  outs axsig*kampenv, aysig*kampenv

endin

;---------------------------------------------------------------------------
; 4 Pole Low Pass Resonant Filter
;---------------------------------------------------------------------------
        instr   20

ifcoch  =       p4   ; Cut-Off Frequency
irezch  =       p5   ; Resonance
irzmix  =       p6   ; Resonance Mix
izin    =       p7   ; Input channel
izout   =       p8   ; Output channel

kfco    zkr     ifcoch              ; Cut-off Frequency envelope from channel
kfcort  =       sqrt(kfco)          ; Needed for the filter
krezo   zkr     irezch              ; Resonance envelope from channel
krez    =       krezo*kfco/500      ; Add more resonance at high Fco
kdclick linseg  0, .002, 1, p3-.004, 1, .002, 0  ; Declick


axn     zar     izin                ; Read input channel

ka1     =       1000/krez/kfco-1    ; Compute filter coeff. a1
ka2     =       100000/kfco/kfco    ; Compute filter coeff. a2
kb      =       1+ka1+ka2           ; Compute filter coeff. b
ay1     nlfilt  axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1  ; Use the non-linear filter
ay      nlfilt  ay1/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1  ; as an ordinary filter.

ka1lp   =       1000/kfco-1         ; Resonance of 1 is a low pass filter
ka2lp   =       100000/kfco/kfco
kblp    =       1+ka1lp+ka2lp
ay1lp   nlfilt  axn/kblp, (ka1lp+2*ka2lp)/kblp, -ka2lp/kblp, 0, 0, 1   ; Low-pass filter
aylp    nlfilt  ay1lp/kblp, (ka1lp+2*ka2lp)/kblp, -ka2lp/kblp, 0, 0, 1

ayrez   =       ay - aylp    ; Extract the resonance part.
ayrz    =       ayrez/kfco   ; Use lower amplitudes at higher Fco

ay2     =       aylp*6*(1-irzmix) + ayrz*300*irzmix  ; Scale the low pass and resonance separately

       zaw      ay2*kdclick, izout   ; Write to the output channel

       endin

;---------------------------------------------------------------------------
; RMS
;---------------------------------------------------------------------------
          instr  23

ifqc      =      1/p4     ; RMS calculation frequency
iinch     =      p5       ; Input Channel
ioutch    =      p6       ; Output Channel

asig      zar        iinch                 ; Read input channel
kamp      rms        asig, ifqc            ; Find rms level

;aout      =          kamp
;          outs       aout*10000, aout*10000

kampo     ktone      kamp, ifqc/2
          zkw        kampo, ioutch

          endin

;---------------------------------------------------------------------------
; Table Audio Oscillator
;---------------------------------------------------------------------------
          instr   24

itab1     init    p4
ifqch     init    p5
iout      init    p6

kfqc      zkr     ifqch
asin1     oscil   1, kfqc, itab1  ;Oscillator

          zawm    asin1, iout           ;Mix to output channel

endin

;---------------------------------------------------------------------------
; Contolled Amplifier
;---------------------------------------------------------------------------
        instr   25

imaxch  =       p4
igainch =       p5
inch    =       p6
ioutch  =       p7

kmax    zkr     imaxch
kgain   zkr     igainch
asig    zar     inch

aout    =       kmax*kgain*asig

        zaw     aout, ioutch

        endin

;---------------------------------------------------------------------------
; Distortion
;---------------------------------------------------------------------------
         instr   30

igaini   =       p4          ; Pre gain
igainf   =       p5          ; Post gain
iduty    =       p6          ; Duty cycle offset
itab1    =       p7          ; Distortion Table
izin     =       p8          ; Input channel
izout    =       p9          ; Output channel

kamp     linseg  0, .002, 1, p3-.004, 1, .002, 0   ; Declick

asig     zar     izin                  ; Read input channel
asign    =       igaini*asig/60000     ; Normalize the signal
aclip    tablei  asign,itab1,1,.5      ; Read the waveshaping table
aclip    =       igainf*aclip*15000    ; Re-amplify the signal

atemp    delayr  .001                  ; Amplitude and slope based delay
aout     deltapi (2-iduty*asign)/15000
         delayw  aclip

         zaw     aout, izout           ; Write to output channel

         endin

;---------------------------------------------------------------------------
; Flanger
;---------------------------------------------------------------------------
instr 40

irate    =       p4          ; Flanger rate
idepth   =       p5/1000     ; Flanger depth time converted to msec.
iwave    =       p6          ; Waveshape
ifdbk    =       p7          ; Amount of feedback
imixfl   =       p8          ; Mix of flanged signal
imix     =       1-imixfl    ; Mix of direct signal
ideloff  =       p9/1000     ; Delay offset
iphase   =       p10         ; Phase shift
izin     =       p11         ; Input channel
izout    =       p12         ; Output channel
adel1    init    0           ; Must be initialized to zero for feedback

kamp     linseg  0, .002, 1, p3-.004, 1, .002, 0  ; Declick

asig     zar     izin        ; Read input channel

asig1    =       asig+ifdbk*adel1       ; Add feedback to signal
aosc1    oscil   idepth, irate, iwave, iphase  ; Flanger LFO
aosc2    =       aosc1+ideloff          ; Add LFO to offset

atemp    delayr  idepth+ideloff         ; Total delay time
adel1    deltapi aosc2                  ; Variable delay tap
         delayw  asig1                  ; Write to delay line

aout     =       imixfl*adel1+imix*asig ; Mix direct and flanged signals
         zaw     aout, izout            ; Write to output channel

         endin

;---------------------------------------------------------------------------
; Mixer Section
;---------------------------------------------------------------------------
          instr 100

asig1     zar   p4             ; Read input channel 1
igl1      init  p5*p6          ; Left gain
igr1      init  p5*(1-p6)      ; Right gain

asig2     zar   p7             ; Read input channel 2
igl2      init  p8*p9          ; Left gain
igr2      init  p8*(1-p9)      ; Right gain

kdclick   linseg  0, .001, 1, p3-.002, 1, .001, 0  ; Declick

asigl     =     asig1*igl1 + asig2*igl2  ; Scale and sum
asigr     =     asig1*igr1 + asig2*igr2

          outs  kdclick*asigl, kdclick*asigr   ; Output stereo pair

          endin

; Clear audio
          instr 110

          zacl  0, 30          ; Clear audio channels 0 to 30

          endin
