; A Set of Band Limited Instruments with a Resonant Filter
; by Hans Mikelson 11/20/97
; partially derived from code by Josep M Comajuncosas-Csound & Tim Stilson-CCRMA

sr = 44100
kr = 22050
ksmps = 2
nchnls=2
zakinit 30, 30

; Low Frequency Oscillator
       instr    1

iamp   =        p4
ilfo   =        p5
iwave  =        p6
ioutch =        p7

klfo   oscil    iamp, ilfo, iwave
       zkw      klfo, ioutch

endin

; Envelope
       instr    2

idur   =        p3
iamp   =        p4
ishape =        p5
ioutch =        p6

kenv   oscil    iamp, 1/idur, ishape
       zkw      kenv, ioutch

       endin

; Pulse Width Modulation (Band Limited Impulse Train)
        instr   10

idur    =       p3
iamp    =       p4
ifqc    =       cpspch(p5)
iampenv =       p6
ifcotab =       p7
ireztab =       p8
ifcoch  =       p9
irezch  =       p10
ilfoch  =       p11

kdclik linseg   0, .002, iamp, idur-.004, iamp, .002, 0
kamp   oscil    kdclik, 1/idur, iampenv 
kfcoe  oscil    1, 1/idur, ifcotab
kreze  oscil    1, 1/idur, ireztab
kfcoc  zkr      ifcoch
krezc  zkr      irezch
kfco   =        kfcoe*kfcoc
krez   =        kreze*krezc

klfo   zkr      ilfoch
kfqc   =        ifqc+klfo
kfco   oscil    1, 1/idur, ifcotab
krez   oscil    1, 1/idur, ireztab

apulse1 buzz    1,ifqc, sr/2/ifqc, 1 ; Avoid aliasing
apulse3 buzz    1,kfqc, sr/2/kfqc, 1
apulse2 delay   apulse3, 1/ifqc/2         ; a better idea?
avpw    =       apulse1 - apulse2         ; two inverted pulses at variable distance
apwmdc  integ   avpw
axn     =       apwmdc-.5                ; remove DC offset caused by integ

; Resonant Lowpass Filter (4 Pole)
ka1    =        1000/krez/kfco-1
ka2    =        100000/kfco/kfco
kb     =        1+ka1+ka2
ayn    nlfilt   axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1
ayn2   nlfilt   ayn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1

       outs     kamp*ayn2, kamp*ayn2
       endin

; Rectangle Wave (Band Limited Impulse Train)
        instr   11

idur    =       p3
iamp    =       p4
ifqc    =       cpspch(p5)
iampenv =       p6
ifcotab =       p7
ireztab =       p8
ifcoch  =       p9
irezch  =       p10
ipw     =       p11

kdclik linseg   0, .002, iamp, idur-.004, iamp, .002, 0
kamp   oscil    kdclik, 1/idur, iampenv 
kfcoe  oscil    1, 1/idur, ifcotab
kreze  oscil    1, 1/idur, ireztab
kfcoc  zkr      ifcoch
krezc  zkr      irezch
kfco   =        kfcoe*kfcoc
krez   =        kreze*krezc

kfco   oscil    1, 1/idur, ifcotab
krez   oscil    1, 1/idur, ireztab

apulse1 buzz    1,ifqc, sr/2/ifqc, 1 ; Avoid aliasing
apulse3 buzz    1,ifqc, sr/2/ifqc, 1
apulse2 delay   apulse3, 1/ifqc*ipw         ; a better idea?
avpw    =       apulse1 - apulse2         ; two inverted pulses at variable distance
apwmdc  integ   avpw
axn     =       apwmdc-.5                 ; remove DC offset caused by integ

; Resonant Lowpass Filter (4 Pole)
ka1    =        1000/krez/kfco-1
ka2    =        100000/kfco/kfco
kb     =        1+ka1+ka2
ayn    nlfilt   axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1
ayn2   nlfilt   ayn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1

       outs     kamp*ayn2, kamp*ayn2
       endin

; Sawtooth (Band Limited Impulse Train)
        instr   12

idur    =       p3
iamp    =       p4
ifqc    =       cpspch(p5)
iampenv =       p6
ifcotab =       p7
ireztab =       p8
ifcoch  =       p9
irezch  =       p10

kdclik linseg   0, .002, iamp, idur-.004, iamp, .002, 0
kamp   oscil    kdclik, 1/idur, iampenv 
kfcoe  oscil    1, 1/idur, ifcotab
kreze  oscil    1, 1/idur, ireztab
kfcoc  zkr      ifcoch
krezc  zkr      irezch
kfco   =        kfcoe*kfcoc
krez   =        kreze*krezc

apulse buzz     1,ifqc, sr/2/ifqc, 1 ; Avoid aliasing
asawdc integ    apulse
axn    =        asawdc-.5

; Resonant Lowpass Filter (4 Pole)
ka1    =        1000/krez/kfco-1
ka2    =        100000/kfco/kfco
kb     =        1+ka1+ka2
ayn    nlfilt   axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1
ayn2   nlfilt   ayn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1

       outs     kamp*ayn2, kamp*ayn2
       endin

; Sawtooth Oscillator Sync (Band Limited Impulse Train)
; Can't get this to work.  It basically ouputs a chord.
        instr   13

idur    =       p3
iamp    =       p4
ifqcm   =       cpspch(p5)
ifqcs   =       cpspch(p6)
iampenv =       p7
ifcotab =       p8
ireztab =       p9
ifcoch  =       p10
irezch  =       p11
asaw1   init    0
asaw2   init    0

kdclik linseg   0, .002, iamp, idur-.004, iamp, .002, 0
kamp   oscil    kdclik, 1/idur, iampenv 
kfcoe  oscil    1, 1/idur, ifcotab
kreze  oscil    1, 1/idur, ireztab
kfcoc  zkr      ifcoch
krezc  zkr      irezch
kfco   =        kfcoe*kfcoc
krez   =        kreze*krezc

apuls1 buzz     1,ifqcm, sr/2/ifqcm, 1 ; Avoid aliasing
apuls2 buzz     1,ifqcs, sr/2/ifqcs, 1
asaw1  integ    apuls1
asaw2  integ    apuls2

asawos integ    apuls1+apuls2
;asawos =        asawos+(apuls1-.25)*(1-asaw1)+(apuls2-.25)*(1-asaw2)
axn    =        asawos-1

; Resonant Lowpass Filter (4 Pole)
ka1    =        1000/krez/kfco-1
ka2    =        100000/kfco/kfco
kb     =        1+ka1+ka2
ayn    nlfilt   axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1
ayn2   nlfilt   ayn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1

       outs     kamp*axn, kamp*axn
       endin

; Ramp/Saw Modulation (Band Limited Impulse Train) Levels need adjusting
        instr   14

idur    =       p3
iamp    =       p4
ifqc    =       cpspch(p5)
iampenv =       p6
ifcotab =       p7
ireztab =       p8
ifcoch  =       p9
irezch  =       p10
ilfoch  =       p11

kdclik linseg   0, .002, iamp, idur-.004, iamp, .002, 0
kamp   oscil    kdclik, 1/idur, iampenv 
kfcoe  oscil    1, 1/idur, ifcotab
kreze  oscil    1, 1/idur, ireztab
kfcoc  zkr      ifcoch
krezc  zkr      irezch
kfco   =        kfcoe*kfcoc
krez   =        kreze*krezc

klfo   zkr      ilfoch
kfqc   =        ifqc+klfo
kfco   oscil    1, 1/idur, ifcotab
krez   oscil    1, 1/idur, ireztab

apulse1 buzz    1,ifqc, sr/2/ifqc, 1      ; Avoid aliasing
apulse3 buzz    1,kfqc, sr/2/kfqc, 1
apulse2 delay   apulse3, 1/ifqc/2         ; a better idea?
avpw    =       apulse1 - apulse2         ; two inverted pulses at variable distance
apwmdc  integ   avpw
apwm    butterhp apwmdc,5                 ; remove DC offset caused by integ
asawrmp integ   apwm
asrnorm =       2*ifqc/sr/klfo/1.5/(1-klfo/1.5)/100
axn     butterhp asrnorm,10

; Resonant Lowpass Filter (4 Pole)
ka1    =        1000/krez/kfco-1
ka2    =        100000/kfco/kfco
kb     =        1+ka1+ka2
ayn    nlfilt   axn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1
ayn2   nlfilt   ayn/kb, (ka1+2*ka2)/kb, -ka2/kb, 0, 0, 1

       outs     kamp*axn, kamp*asawrmp
       endin


