sr=44100
kr=4410
ksmps=10
nchnls=2

;---------------------------------------------------------------------------
; Sample with Noise Filter Resonance
;---------------------------------------------------------------------------
         instr  1

idur     =      p3                 ; Duration
iamp     =      p4                 ; Amplitude
irate    =      p5                 ; Read rate
isndin   =      p6                 ; Sound input file
ipantab  =      p7                 ; Pan table
krez     =      p8                 ; Resonance
iband    =      0                  ; Just leave it zero...
ifqcadj  =      .149659863*sr      ; Adjustment for filter frequency

kfco1   expseg  800, idur/5, 5000, idur/5, 1000, idur/5, 2000, idur/5, 700, idur/5, 3000 ; Freq Cut-Off
klfo    oscil   1, 20, 20
kfco    =       kfco1*klfo

ain1, ain2 diskin isndin, irate    ; Read sound file

axn     =  ain1+ain2               ; Mix the two together

; Resonant Lowpass Filter (4 Pole)
kc     =        ifqcadj/kfco                                 ; Filter constant
krez2  =        krez/(1+exp(kfco/11000))                     ; Adjust for high frequencies
ka1    =        kc/krez2-(1+krez2*iband)                     ; Adjust for band pass character
kasq    =       kc*kc                                        ; c^2
kb     =        1+ka1+kasq                                   ; Find b

ayn    nlfilt   axn/kb, (ka1+2*kasq)/kb, -kasq/kb, 0, 0, 1   ; Use the non-linear filter linearly
ayn2   nlfilt   ayn/kb, (ka1+2*kasq)/kb, -kasq/kb, 0, 0, 1   ; 4-Pole

; Resonant Lowpass Filter (4 Pole)
kcl    =        ifqcadj/kfco
krez2l =        2.0/(1+exp(kfco/11000))                      ; Same as above but low resonance this time.
ka1l   =        kcl/krez2l-(1+krez2l*iband)
kasql  =        kcl*kcl
kbl    =        1+ka1l+kasql

aynl   nlfilt   axn/kbl, (ka1l+2*kasql)/kbl, -kasql/kbl, 0, 0, 1
ayn2l  nlfilt   aynl/kbl, (ka1l+2*kasql)/kbl, -kasql/kbl, 0, 0, 1

ares1  =       ayn2-ayn2l                    ; Isolate the resonance
krms   rms     ares1, 100                    ; Compute the average resonance level

arand1 rand     krms                         ; Noise source
ares2  butterbp arand1, kfco, kfco/4         ; Band filter it based on filter cut-off
ares3  butterbp arand1, kfco*2, kfco/4       ; Get another band
aout   =       ayn2l+(ares2+ares3)           ; Add back on to the low-pass signal

kpan   oscil   1, 1/idur, ipantab            ; Panning
kpanl  =       sqrt(kpan)*iamp
kpanr  =       sqrt(1-kpan)*iamp
       outs    aout*kpanl, aout*kpanr        ; Give it to me.

       endin

;---------------------------------------------------------------------------
; Sample with FM Filter Resonance
;---------------------------------------------------------------------------
         instr  2

idur     =      p3                    ; Duration
iamp     =      p4                    ; Amplitude
irate    =      p5                    ; Read rate
isndin   =      p6                    ; Sound input file number
ipantab  =      p7                    ; Panning table
krez     =      p8                    ; Resonance
iband    =      0                     ; Controls band-pass character of the filter but
ifqcadj  =      .149659863*sr         ; values other than zero make the filter unstable

kfco1   expseg  5000, idur/5, 1000, idur/5, 2000, idur/5, 700, idur/5, 3000, idur/5, 800 ; Filter sweep
klfo    oscil   1, 20, 20
kfco    =       kfco1*klfo

ain1, ain2 diskin isndin, irate       ; Read in the sound file (must be stereo)

axn     =  ain1+ain2                  ; Just mix together

; Resonant Lowpass Filter (4 Pole)
kc     =        ifqcadj/kfco                                  ; Find the mysterious constant "C"
krez2  =        krez/(1+exp(kfco/11000))                      ; Reduce res at high frequencies
ka1    =        kc/krez2-(1+krez2*iband)                      ; Compute A1
kasq    =       kc*kc                                         ; C^2
kb     =        1+ka1+kasq                                    ; Find B

ayn    nlfilt   axn/kb, (ka1+2*kasq)/kb, -kasq/kb, 0, 0, 1    ; Use nlfilt for filter sweep.
ayn2   nlfilt   ayn/kb, (ka1+2*kasq)/kb, -kasq/kb, 0, 0, 1    ; Do it again for four pole.

; Resonant Lowpass Filter (4 Pole)                            ; Same as above but low-pass without resonance.
kcl    =        ifqcadj/kfco
krez2l =        2.0/(1+exp(kfco/11000))
ka1l   =        kcl/krez2l-(1+krez2l*iband)
kasql  =        kcl*kcl
kbl    =        1+ka1l+kasql

aynl   nlfilt   axn/kbl, (ka1l+2*kasql)/kbl, -kasql/kbl, 0, 0, 1
ayn2l  nlfilt   aynl/kbl, (ka1l+2*kasql)/kbl, -kasql/kbl, 0, 0, 1

ares1  =       ayn2-ayn2l                                  ; Extract resonance
krms   rms     ares1, 100                                  ; How big is the resonance
;              Amp   Fqc       Car  Mod      MAmp  Wave
ares2  foscil  krms, kfco,     1,   .75,     3,    1       ; Generate an FM signal based on resonance
aout   =       ayn2l+ares2/4                               ; Add it on to the low-pass version

kpan   oscil   1, 1/idur, ipantab                          ; Handle equal power panning
kpanl  =       sqrt(kpan)*iamp
kpanr  =       sqrt(1-kpan)*iamp
       outs    aout*kpanl, aout*kpanr                      ; Output

       endin

